function langtonsAnt(varargin)
%langtonsAnt by Sebastian Weichwald [2011-10-25-19]
%
% A Simulation of Langton's Ant
% 
% 1 - call of langtonsAnt function
% - - - - - - - - - - - - - - - - -
%     langtonsAnt() - simulation of Langton's Ant with default values
%
%     langtonsAnt('option', value) - the following options can be set
%
%         boundaryCondition - 'continuous'           - 0
%                           - 'impenetrable'         - 1
%                           - 'impenetrableHBorders' - 2
%                           - 'impenetrableVBorders' - 3
%
%         grid - logical matrix - 'true' represents a white square
%                                 'false' represents a black square
%              - string - is one of the following predefined grids
%                       - 'checkerboard'
%                       - 'dotted'
%                       - 'square'
%                       - 'stripes'
%
%         initialDir - '^' - 0 - up
%                    - '>' - 1 - right
%                    - 'v' - 2 - down
%                    - '<' - 3 - left
%
%         initialPos - 1-by-2 matrix with finite positive integer elements
%                          greater than 0 and smaller than gridX and gridY
%                          the first element specifies the X-coordinate
%                          and the second element gives the Y-coordinate
%                          of the ant's initial position
%                    - NaN - initialPos is set to the center of the grid
%
%         slowMotionDelay - finite positive numeric
%                           used in slowMotionModes 1, 3 and 5
%
%         slowMotionSnapshotSteps - finite positive integer greater than 0
%                                   used in slowMotionMode 1
%
%         slowMotionMode - 'fastest'      - 0 - plot after all computations
%                                               are finished
%                        - 'snapshots'    - 1 - plot after number of steps
%                                               specified by
%                                               slowMotionSnapshotSteps and
%                                               pause afterwards for the
%                                               number of seconds specified
%                                               by slowMotionDelay
%                        - 'fastSteps'    - 2 - plot every step afap
%                        - 'steps'        - 3 - plot every step and pause
%                                               afterwards for number of
%                                               seconds specified by
%                                               slowMotionDelay
%                        - 'fastMovement' - 4 - plot every movement
%                                               (step, flip square, turn)
%                        - 'movement'     - 5 - plot every movement and
%                                               pause afterwards for number
%                                               of seconds specified by
%                                               slowMotionDelay
%
%         steps - finite positive integer
%
%     Additional features during simulation:
%         - mouseclick on the figure window pauses/resumes the simulation
%         - while the figure window is focused a key press speeds up the
%           simulation by decreasing the number of slowMotionMode
%
%
% 2 - description of langtonsAnt function
% - - - - - - - - - - - - - - - - - - - -
% Langton's Ant moves around a 2-dimensional M-by-N grid.
% Each square is either black (false) or white (true).
%
% While it's moving along the grid it follows the following two rules:
%
% a) If it is on a white square
%     it flips the square's colour to black
%     turns right
%     moves one square forward
%
% b) If it is on a black square
%     it flips the square's colour to white
%     turns left
%     moves one square forward
%
% The ant is represented by a red triangle (see following symbols)
% according to one of the four directions it's facing to:
%     ^ - 0 - up
%     > - 1 - right
%     v - 2 - down
%     < - 3 - left
%
% Langton's Ant's grid can have one of the following boundary conditions:
%
% 0 continuous: the grid is continuous:
%     the left and right sides meet and the top and bottom edges meet
%
% 1 impenetrable: the grid has impenetrable borders:
%     if the ant's next step would require the ant to cross the border
%     the ant turns back to the direction it faced to before and continues
%     from there according to the rules and the current square's colour
%
% 2 impenetrableHborders: the grid has impenetrable horizontal borders:
%     the top and bottom borders are impenetrable
%     the left and right sides meet
%
% 3 impenetrableVBorders: the grid has impenetrable vertical borders:
%     the left and right borders are impenetrable
%     the top and bottom edges meet





%% - - - default options - - - %
% struct array with default options
options = struct('boundaryCondition', 0, ... % 'continuous'
                 'grid', ones(100), ...
                 'gridCheckerboard', zeros(100), ...
                 'gridDotted', zeros(100), ...
                 'gridSquare', zeros(100), ...
                 'gridStripes', zeros(100), ...
                 'initialDir', 0, ... % '^'
                 'initialPos', NaN, ...
                 'slowMotionDelay', .2, ...
                 'slowMotionSnapshotSteps', 100, ...
                 'slowMotionMode', 1, ... % 'fastest'
                 'steps', 12500);

% generate predefined grids
tmp=repmat( [ ones(50,1) zeros(50,1) ], 1, 25);

options.gridDotted=tmp - rot90(tmp);

tmp=repmat( [ zeros(80, 10) ones(80, 10) ], 1, 4);

options.gridCheckerboard=tmp - rot90(tmp);

options.gridStripes=tmp;

clear tmp;

options.gridSquare=ones(100);
options.gridSquare(25:75,25)=false;
options.gridSquare(25:75,75)=false;
options.gridSquare(25,25:75)=false;
options.gridSquare(75,25:75)=false;





%% - - - parse varargin - - - %
% nargin must be a multiple of 2 since we have only option-value pairs
if mod(nargin,2) == 1
    error([ 'Wrong number of input arguments. ' ...
            'Each option must be given as option-value pair.' ]);
end

% get option-value pairs set in varargin
for i=1:2:nargin
    switch varargin{i}
        
        % validate boundaryCondition
        case 'boundaryCondition'
            if ischar(varargin{i+1})
                switch varargin{i+1}
                    case 'continuous'
                        options.boundaryCondition=0;
                    case 'impenetrable'
                        options.boundaryCondition=1;
                    case 'impenetrableHBorders'
                        options.boundaryCondition=2;
                    case 'impenetrableVBorders'
                        options.boundaryCondition=3;
                    otherwise
                        error([ 'boundaryCondition must be one of the ' ...
                                'following strings: ''continuous'', ' ...
                                '''impenetrable'', ' ...
                                '''imepenetrableHBorders'', ' ...
                                '''impenetrableVBorders''' ]);
                end
            else
                error([ 'boundaryCondition must be one of the ' ...
                        'following strings: ''continuous'', ' ...
                        '''impenetrable'', ''imepenetrableHBorders'', ' ...
                        '''impenetrableVBorders''' ]);
            end

        % validate grid
        case 'grid'
            if islogical(varargin{i+1}) && ~isempty(varargin{i+1})
                options.grid=varargin{i+1};
            elseif ischar(varargin{i+1})
                switch varargin{i+1}
                    case 'checkerboard'
                        options.grid=options.gridCheckerboard;
                    case 'dotted'
                        options.grid=options.gridDotted;
                    case 'square'
                        options.grid=options.gridSquare;
                    case 'stripes'
                        options.grid=options.gridStripes;
                    otherwise
                        error([ 'there are only the following ' ...
                                'predefined grids: ''checkerboard'', ' ...
                                '''dotted'', ''square'', ''stripes''' ]);
                end
            else
                error([ 'grid must be either a logical grid or one of ' ...
                        'the following strings: ''checkerboard'', ' ...
                        '''dotted'', ''square'', ''stripes''' ]);
            end

        % validate initialDir
        case 'initialDir'
            if ischar(varargin{i+1})
                switch varargin{i+1}
                    case '^'
                        options.initialDir=0;
                    case '>'
                        options.initialDir=1;
                    case 'v'
                        options.initialDir=2;
                    case '<'
                        options.initialDir=3;
                    otherwise
                        error([ 'initialDir must be one of the ' ...
                                'following characters: ''^'', ''>'', ' ...
                                '''v'', ''<''' ]);
                end
            else
                error([ 'initialDir must be one of the following ' ...
                        'characters: ''^'', ''>'', ''v'', ''<''' ]);
            end

        % validate initialPos
        % wether initialPos' elements are smaller or equal gridX or gridY
        % can't be validated here, it will during initiation
        case 'initialPos'
            if isnan(varargin{i+1}(1)) ...
                    || ( size(varargin{i+1},1) == 1 ...
                         && size(varargin{i+1},2) == 2 ...
                         && isnumeric(varargin{i+1}) ...
                         && varargin{i+1}(1) > 0 ...
                         && varargin{i+1}(2) > 0 ...
                         && isfinite(varargin{i+1}(1)) ...
                         && isfinite(varargin{i+1}(2)) ...
                         && floor(varargin{i+1}(1)) == ceil(varargin{i+1}(1)) ...
                         && floor(varargin{i+1}(2)) == ceil(varargin{i+1}(2)) )
                options.initialPos=varargin{i+1};
            else
                error([ 'initialPos must be either NaN or a 1-by-2 ' ...
                        'matrix specifying the X- and Y-coordinate ' ...
                        'where the elements are finite positive ' ...
                        'integers greater than 0 and smaller than ' ...
                        'gridX and gridY' ]);
            end

        % validate slowMotionMode
        case 'slowMotionMode'
            if ischar(varargin{i+1})
                switch varargin{i+1}
                    case 'fastest'
                        options.slowMotionMode=0;
                    case 'snapshots'
                        options.slowMotionMode=1;
                    case 'fastSteps'
                        options.slowMotionMode=2;
                    case 'steps'
                        options.slowMotionMode=3;
                    case 'fastMovement'
                        options.slowMotionMode=4;
                    case 'movement'
                        options.slowMotionMode=5;
                    otherwise
                        error([ 'slowMotionMode must be one of the ' ...
                                'following strings: ''fastest'', ' ...
                                '''snapshots'', ''fastSteps'', ' ...
                                '''steps'', ''fastMovement'', ' ...
                                '''movement''' ]);
                end
            else
                error([ 'slowMotionMode must be one of the following ' ...
                        'strings: ''fastest'', ''snapshots'', ' ...
                        '''fastSteps'', ''steps'', ''fastMovement'', ' ...
                        '''movement''' ]);
            end
        
        % validate slowMotionDelay
        case 'slowMotionDelay'
            if ~isnumeric(varargin{i+1}) || varargin{i+1}<0 ...
                    || ~isfinite(varargin{i+1})
                error('slowMotionDelay must be a finite positive numeric');
            end
            options.slowMotionDelay=varargin{i+1};
            
        % validate slowMotionSnapshotSteps
        case 'slowMotionSnapshotSteps'
            if ~isnumeric(varargin{i+1}) || varargin{i+1} <= 0 ...
                    || ~isfinite(varargin{i+1}) ...
                    || floor(varargin{i+1}) ~= ceil(varargin{i+1})
                error([ 'slowMotionSnapshotSteps must be a finite ' ...
                        'positive integer greater than 0' ]);
            end
            options.slowMotionSnapshotSteps=varargin{i+1};

        % validate steps
        case 'steps'
            if ~isnumeric(varargin{i+1}) || varargin{i+1}<0 ...
                    || ~isfinite(varargin{i+1}) ...
                    || floor(varargin{i+1}) ~= ceil(varargin{i+1})
                error('steps must be a finite positive integer');
            end
            options.steps=varargin{i+1};

        otherwise
            error(['Unknown option ''' varargin{i} '''.']);
    end
end

%display options
options





%% - - - initiate - - - %

% grid is logical
data.grid = logical(options.grid);
data.gridX = size(data.grid,2);
data.gridY = size(data.grid,1);

% initialDir
data.dirs = ['^' , '>' , 'v' , '<'];
data.posChangeXDir = [0 1 0 -1];
data.posChangeYDir = [1 0 -1 0];
data.curDir = options.initialDir;

% if initialPos is NaN -> set pos to middle of grid
% otherwise validate initialPos input
if isnan(options.initialPos)
    data.curX = ceil(data.gridX/2);
    data.curY = ceil(data.gridY/2);
else
    if options.initialPos(1) > data.gridX ...
            || options.initialPos(2) > data.gridY
        error([ 'initialPos must be either NaN or a 1-by-2 matrix ' ...
                'specifying the X- and Y-coordinate where the ' ...
                'elements are finite positive integers greater than 0 ' ...
                'and smaller than gridX and gridY' ]);
    end
    data.curX = options.initialPos(1);
    data.curY = options.initialPos(2);
end

% NOTE
% we create two vectors
%     exceedXindexes with gridX+2 elements
%     exceedYindexes with gridY+2 elements
% for every possible data.curPos the vectors contain the
% "actual" ant's x and y values in the grid
% the vectors hold one more element on either side in order to
% "translate" the ant's position if it exceeds the grid's dimensions
% - - -
% This turned out to be faster than checking after every step if the ant
% exceeded the grid's boundaries and correct it's x and y values
% accordingly to the desired boundaryCondition
switch options.boundaryCondition
    case 0
        data.exceedXindexes = [ data.gridX 1:data.gridX 1 ];
        data.exceedYindexes = [ data.gridY 1:data.gridY 1 ];
    case 1
        data.exceedXindexes = [ 1 1:data.gridX data.gridX ];
        data.exceedYindexes = [ 1 1:data.gridY data.gridY ];
    case 2
        data.exceedXindexes = [ data.gridX 1:data.gridX 1 ];
        data.exceedYindexes = [ 1 1:data.gridY data.gridY ];
    case 3
        data.exceedXindexes = [ 1 1:data.gridX data.gridX ];
        data.exceedYindexes = [ data.gridY 1:data.gridY 1 ];
end





%% - - - initiate graphics - - - %
hold on

% plot initial grid and get handler
data.hGrid = image(data.grid);

% squares should really be squares in the ant's world
axis equal

% hide axes
set(gca, 'Visible', 'off')

% only two rgb colours black and white
colormap([0 0 0; 1 1 1]);

% plot ant in initial position and direction and get handler
data.hAnt = plot(data.curX, data.curY, ...
                 [ 'r' data.dirs(data.curDir+1)], ...
                 'MarkerFaceColor', [1 0 0]);

% bring figure to foreground
figure(gcf);

% register langtonsAntPauseState as global flag initially set to 'false'
% register langtonsAntPause function as callback for the figure
global langtonsAntPauseState;
langtonsAntPauseState=false;
set(gcf,'ButtonDownFcn',{@langtonsAntPause});

% register langtonsAntSlowMotionMode as global flag
global langtonsAntSlowMotionMode;
langtonsAntSlowMotionMode = options.slowMotionMode;
set(gcf,'WindowKeyPressFcn',{@langtonsAntChangeSlowMotionMode});

% display data
data





%% - - - start simulation - - - %
langtonsAntDelay(langtonsAntSlowMotionMode, options.slowMotionDelay, true);

for i=1:options.steps

    % paused?
    while langtonsAntPauseState
        pause(.1);
    end

    % slowMotion?
    % for slowMotionMode 'fastest' plot data should be set after all 
    % computations are done, for slowMotionMode 'snapshots' plot data
    % should only be set if mod(i, slowMotionSnapshotSteps) == 0
    setData = langtonsAntSlowMotionMode >= 2 ...
                  || ( langtonsAntSlowMotionMode == 1 ...
                       && mod(i, options.slowMotionSnapshotSteps) == 0) ...
                  || ( i == options.steps );

    % flip colour of current square
    data.grid(data.curY,data.curX)=~data.grid(data.curY,data.curX);
    
    % set changed grid if necessary and possibly pause
    if setData
        set(data.hGrid, 'CData', data.grid);
        langtonsAntDelay(langtonsAntSlowMotionMode, ...
            options.slowMotionDelay, false);
    end

    % change facing
    % CAUTION: square already flipped!
    % if current square is white i. e. 'true'
    % the ant was on a black square before
    % - - -
    % data.dirs = ['^' , '>' , 'v' , '<'];
    if data.grid(data.curY,data.curX)
        data.curDir=data.curDir+2;
    end
    data.curDir=mod(data.curDir+1,4);

    % set changed direction of ant if necessary and possibly pause
    if setData
        set(data.hAnt, 'Marker', data.dirs(data.curDir+1));
        langtonsAntDelay(langtonsAntSlowMotionMode, ...
            options.slowMotionDelay, false);
    end

    % move one square forward
    % and get the actual position with help of data.exceed#indexes
    % so here we're taking care of the boundaryCondition
    tmpX = data.curX;
    tmpY = data.curY;
    data.curX = data.exceedXindexes(data.curX ...
                                    + data.posChangeXDir(data.curDir+1) ...
                                    + 1);
    data.curY = data.exceedYindexes(data.curY ...
                                    + data.posChangeYDir(data.curDir+1) ...
                                    + 1);

    % if ant reached impenetrable borders (the position didn't change)
    % turn back i. e. in the opposite direction as the ant turned above
    % CAUTION: square already flipped!
    % if current square is white i. e. 'true'
    % the ant was on a black square before
    % so above the ant turned left
    % so here the ant needs to turn right
    % - - -
    % data.dirs = ['^' , '>' , 'v' , '<'];
    if options.boundaryCondition > 0 && tmpX == data.curX ...
            && tmpY == data.curY
        if ~data.grid(data.curY,data.curX)
            data.curDir=data.curDir+2;
        end
        data.curDir=mod(data.curDir+1,4);
        
        % set changed direction of ant if necessary and possibly pause
        if setData
            set(data.hAnt, 'Marker', data.dirs(data.curDir+1));
            langtonsAntDelay(langtonsAntSlowMotionMode, ...
                options.slowMotionDelay, false);
        end
    end

    % if there are no impenetrable borders (the position changed) data.curX
    % and data.curY are already set correctly
    
    % set changed position of ant if necessary and possibly pause
    if setData
        set(data.hAnt, 'XData', data.curX, 'YData', data.curY);
        langtonsAntDelay(langtonsAntSlowMotionMode, ...
            options.slowMotionDelay, true);
    end
end





%% - - - finish - - - %
hold off;
display('langtonsAnt finished');